@@ -4,6 +4,8 @@ use thiserror::Error;
 pub enum MyError {
     #[error("Git repo does not exist.")]
     GitRepoDoesNotExist,
+    #[error("Path is not valid.")]
+    InvalidPath,
     #[error("io error")]
     Io {
         #[from]
@@ -26,4 +28,14 @@ pub enum MyError {
         #[from]
         source: std::str::Utf8Error,
     },
+    #[error("Search indexer failed in some way")]
+    SearchIndex {
+        #[from]
+        source: tantivy::TantivyError,
+    },
+    #[error("Failed to parse search query")]
+    SearchQueryParsing {
+        #[from]
+        source: tantivy::query::QueryParserError,
+    },
 }
