@@ -1,9 +1,9 @@
-use pulldown_cmark::{html, Event, HeadingLevel, Options, Parser, Tag};
 use rocket::form::Form;
 use rocket::http::impl_from_uri_param_identity;
 use rocket::http::uri::fmt::Formatter;
 use rocket::http::uri::fmt::Path;
 use rocket::http::uri::fmt::UriDisplay;
+use rocket::http::uri::Origin;
 use rocket::http::uri::Segments;
 use rocket::http::ContentType;
 use rocket::request::FromSegments;
@@ -12,86 +12,14 @@ use rocket::response::Responder;
 use rocket::{Build, Rocket};
 
 use crate::error::MyError;
-use crate::settings::Settings;
-use crate::templates::{render_edit_page, render_page, render_page_placeholder, Breadcrumb};
+use crate::repository;
+use crate::templates;
+use crate::templates::render_search_results;
+use crate::templates::{
+    render_edit_page, render_overview, render_page, render_page_placeholder, Breadcrumb,
+};
 use crate::wiki::Wiki;
 
-struct MarkdownPage<'a> {
-    title: String,
-    events: Vec<Event<'a>>,
-}
-
-fn try_get_h1_title(
-    settings: &Settings,
-    fallback_file_name: &str,
-    events: &mut Vec<Event>,
-) -> String {
-    if settings.h1_title() && events.len() >= 2 {
-        if let Event::Start(Tag::Heading(HeadingLevel::H1, _, _)) = events[0] {
-            let end_ndx = events
-                .iter()
-                .position(|e| matches!(e, Event::End(Tag::Heading(HeadingLevel::H1, _, _))))
-                .unwrap();
-            let title = events[1..end_ndx]
-                .iter()
-                .map(|e| match e {
-                    Event::Text(str) => str.to_string(),
-                    not_str => panic!("Expected str, got: {:?}", not_str),
-                })
-                .collect();
-            events.drain(0..=end_ndx);
-            return title;
-        }
-    }
-    fallback_file_name.to_owned()
-}
-
-impl<'a> MarkdownPage<'a> {
-    fn new(settings: &'a Settings, file_name: &'a str, src: &'a str) -> MarkdownPage<'a> {
-        let mut options = Options::empty();
-        options.insert(Options::ENABLE_TABLES);
-        options.insert(Options::ENABLE_FOOTNOTES);
-        options.insert(Options::ENABLE_STRIKETHROUGH);
-        options.insert(Options::ENABLE_TASKLISTS);
-        options.insert(Options::ENABLE_SMART_PUNCTUATION);
-        let mut events: Vec<Event<'a>> = Parser::new_ext(src, options).collect();
-        let title = try_get_h1_title(settings, file_name, &mut events);
-
-        MarkdownPage { title, events }
-    }
-
-    fn title(&'a self) -> &'a str {
-        &self.title
-    }
-
-    fn render_html(self) -> String {
-        let mut rendered_markdown = String::new();
-        html::push_html(&mut rendered_markdown, self.events.into_iter());
-        rendered_markdown
-    }
-}
-
-fn markdown_response(
-    wiki: &Wiki,
-    edit_url: &str,
-    path: &WikiPagePath,
-    bytes: &[u8],
-) -> Result<String, MyError> {
-    let markdown_input = std::str::from_utf8(bytes)?;
-    let settings = wiki.settings();
-    let markdown_page = MarkdownPage::new(settings, path.file_name().unwrap(), markdown_input);
-
-    let title = markdown_page.title().to_owned();
-    let rendered_markdown = markdown_page.render_html();
-
-    Ok(render_page(
-        &title,
-        edit_url,
-        &rendered_markdown,
-        path.create_breadcrumbs(),
-    )?)
-}
-
 // Most of the time we are returning Page, so it is ok that it is bigger
 #[allow(clippy::large_enum_variant)]
 #[derive(Responder)]
@@ -120,6 +48,12 @@ impl<'r> WikiPagePath<'r> {
         }
     }
 
+    fn append_segment(&self, new_seg: &'r str) -> Self {
+        let mut segments = self.segments.clone();
+        segments.push(new_seg);
+        WikiPagePath { segments }
+    }
+
     fn directories(&self) -> &[&'r str] {
         match self.segments.split_last() {
             Some((_, dirs)) => dirs,
@@ -136,15 +70,19 @@ impl<'r> WikiPagePath<'r> {
         Some(self.file_name_and_extension()?.0)
     }
 
+    #[cfg(test)]
     fn file_extension(&self) -> Option<&str> {
         Some(self.file_name_and_extension()?.1)
     }
 
-    fn create_breadcrumbs(&self) -> Vec<Breadcrumb<'r>> {
-        let mut dirs = self.directories();
+    fn breadcrumbs_helper<F: Fn(&'r [&'r str]) -> Origin>(
+        &'r self,
+        mut dirs: &'r [&'r str],
+        uri_func: F,
+    ) -> Vec<Breadcrumb<'r>> {
         let mut ret = Vec::with_capacity(dirs.len());
         while let Some((name, next_dirs)) = dirs.split_last() {
-            let url = uri!(page(WikiPagePath::from_slice(dirs))).to_string();
+            let url = uri_func(dirs).to_string();
             ret.push(Breadcrumb::new(name, url));
             dirs = next_dirs;
         }
@@ -152,6 +90,18 @@ impl<'r> WikiPagePath<'r> {
         ret.reverse();
         ret
     }
+
+    fn page_breadcrumbs(&'r self) -> Vec<Breadcrumb<'r>> {
+        self.breadcrumbs_helper(self.directories(), |dirs| {
+            uri!(page(WikiPagePath::from_slice(dirs)))
+        })
+    }
+
+    fn overview_breadcrumbs(&'r self) -> Vec<Breadcrumb<'r>> {
+        self.breadcrumbs_helper(&self.segments, |dirs| {
+            uri!(overview(WikiPagePath::from_slice(dirs)))
+        })
+    }
 }
 
 impl<'r> FromSegments<'r> for WikiPagePath<'r> {
@@ -226,53 +176,67 @@ fn edit_view(path: WikiPagePath, w: Wiki) -> Result<response::content::Html<Stri
         &post_url.to_string(),
         &view_url.to_string(),
         content,
-        path.create_breadcrumbs(),
+        path.page_breadcrumbs(),
     )?;
     Ok(response::content::Html(html))
 }
 
-#[get("/page/<path..>")]
-fn page(path: WikiPagePath, w: Wiki) -> WikiPageResponder {
+fn page_response(
+    page: crate::page::Page,
+    path: &WikiPagePath,
+) -> Result<response::content::Html<String>, MyError> {
+    let edit_url = uri!(edit_view(path)).to_string();
+    let html = render_page(&page.title, &edit_url, &page.body, path.page_breadcrumbs())?;
+    let content_typed = response::content::Html(html);
+    Ok(content_typed)
+}
+
+fn page_inner(path: WikiPagePath, w: Wiki) -> Result<WikiPageResponder, MyError> {
     match w.read_file(&path.segments) {
         Ok(bytes) => {
-            let file_extension = path.file_extension();
-            if file_extension == Some("md") {
-                let edit_url = uri!(edit_view(&path)).to_string();
-                return WikiPageResponder::Page(response::content::Html(
-                    markdown_response(&w, &edit_url, &path, &bytes).unwrap(),
-                ));
-            }
-            match file_extension.and_then(ContentType::from_extension) {
-                Some(ext) => WikiPageResponder::TypedFile(response::content::Custom(ext, bytes)),
+            let file_info = path.file_name_and_extension();
+            Ok(match file_info {
+                Some((file_stem, file_ext)) => {
+                    match crate::page::get_page(file_stem, file_ext, &bytes, w.settings())? {
+                        Some(page_model) => {
+                            WikiPageResponder::Page(page_response(page_model, &path)?)
+                        }
+                        None => match ContentType::from_extension(file_ext) {
+                            Some(mine_type) => WikiPageResponder::TypedFile(
+                                response::content::Custom(mine_type, bytes),
+                            ),
+                            None => WikiPageResponder::File(bytes),
+                        },
+                    }
+                }
                 None => WikiPageResponder::File(bytes),
-            }
+            })
         }
         Err(_) => {
             if w.directory_exists(&path.segments).unwrap() {
-                let mut segments = path.segments;
-                let file_name = &format!("{}.md", w.settings().index_page());
-                segments.push(file_name);
-                let path = WikiPagePath::new(segments);
-                WikiPageResponder::Redirect(response::Redirect::to(uri!(page(path))))
+                let file_name = format!("{}.md", w.settings().index_page());
+                let path = path.append_segment(&file_name);
+                Ok(WikiPageResponder::Redirect(response::Redirect::to(uri!(
+                    page(path)
+                ))))
             } else {
                 match path.file_name_and_extension() {
                     Some((file_stem, "md")) => {
                         let create_url = uri!(edit_view(&path));
-                        WikiPageResponder::PagePlaceholder(response::status::NotFound(
-                            response::content::Html(
+                        Ok(WikiPageResponder::PagePlaceholder(
+                            response::status::NotFound(response::content::Html(
                                 render_page_placeholder(
                                     file_stem,
                                     &path.to_string(),
                                     &create_url.to_string(),
-                                    path.create_breadcrumbs(),
+                                    path.page_breadcrumbs(),
                                 )
                                 .unwrap(),
-                            ),
+                            )),
                         ))
                     }
-                    _ => WikiPageResponder::NotFound(response::status::NotFound(format!(
-                        "File not found: {}",
-                        path
+                    _ => Ok(WikiPageResponder::NotFound(response::status::NotFound(
+                        format!("File not found: {}", path),
                     ))),
                 }
             }
@@ -280,6 +244,75 @@ fn page(path: WikiPagePath, w: Wiki) -> WikiPageResponder {
     }
 }
 
+#[get("/page/<path..>")]
+fn page(path: WikiPagePath, w: Wiki) -> Result<WikiPageResponder, MyError> {
+    page_inner(path, w)
+}
+
+fn overview_inner(path: WikiPagePath, w: Wiki) -> Result<response::content::Html<String>, MyError> {
+    let mut entries = w.enumerate_files(&path.segments)?;
+    entries.sort();
+    let entries = entries;
+
+    let directories = entries
+        .iter()
+        .filter_map(|e| match e {
+            repository::RepositoryItem::Directory(name) => {
+                let url = uri!(overview(path.append_segment(name))).to_string();
+                Some(templates::DirectoryEntry::new(name, url))
+            }
+            _ => None,
+        })
+        .collect();
+
+    let files = entries
+        .iter()
+        .filter_map(|e| match e {
+            repository::RepositoryItem::File(name) => {
+                let url = uri!(page(path.append_segment(name))).to_string();
+                Some(templates::DirectoryEntry::new(name, url))
+            }
+            _ => None,
+        })
+        .collect();
+
+    let html = render_overview("Overview", path.overview_breadcrumbs(), directories, files)?;
+    Ok(response::content::Html(html))
+}
+
+#[get("/overview/<path..>")]
+fn overview(path: WikiPagePath, w: Wiki) -> Result<response::content::Html<String>, MyError> {
+    overview_inner(path, w)
+}
+
+fn search_inner(
+    q: &str,
+    offset: Option<usize>,
+    w: Wiki,
+) -> Result<response::content::Html<String>, MyError> {
+    const RESULTS_PER_PAGE: usize = 10;
+    let results = w.search(q, RESULTS_PER_PAGE, offset)?;
+    let prev_url = offset.and_then(|v| {
+        if v >= RESULTS_PER_PAGE {
+            Some(uri!(search(q, Some(v - RESULTS_PER_PAGE))).to_string())
+        } else {
+            None
+        }
+    });
+    let next_url = Some(uri!(search(q, Some(offset.unwrap_or(0) + RESULTS_PER_PAGE))).to_string());
+    let html = render_search_results(q, results, prev_url, next_url)?;
+    Ok(response::content::Html(html))
+}
+
+#[get("/search?<q>&<offset>")]
+fn search(
+    q: &str,
+    offset: Option<usize>,
+    w: Wiki,
+) -> Result<response::content::Html<String>, MyError> {
+    search_inner(q, offset, w)
+}
+
 #[get("/")]
 fn index(w: Wiki) -> response::Redirect {
     let file_name = format!("{}.md", w.settings().index_page());
@@ -288,43 +321,16 @@ fn index(w: Wiki) -> response::Redirect {
 }
 
 pub fn mount_routes(rocket: Rocket<Build>) -> Rocket<Build> {
-    rocket.mount("/", routes![page, edit_save, edit_view, index])
+    rocket.mount(
+        "/",
+        routes![page, search, edit_save, edit_view, overview, index],
+    )
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
 
-    #[test]
-    fn test_normal_title() {
-        let settings = Settings::new("Home", false);
-        let input = "# First H1\n# Second H1";
-        let markdown_page = MarkdownPage::new(&settings, "file_name", &input);
-        assert_eq!("file_name", markdown_page.title());
-        let rendered = markdown_page.render_html();
-        assert_eq!("<h1>First H1</h1>\n<h1>Second H1</h1>\n", rendered);
-    }
-
-    #[test]
-    fn test_h1_title() {
-        let settings = Settings::new("Home", true);
-        let input = "# First H1\n# Second H1";
-        let markdown_page = MarkdownPage::new(&settings, "file_name", &input);
-        assert_eq!("First H1", markdown_page.title());
-        let rendered = markdown_page.render_html();
-        assert_eq!("<h1>Second H1</h1>\n", rendered);
-    }
-
-    #[test]
-    fn test_h1_title_complicated() {
-        let settings = Settings::new("Home", true);
-        let input = "# Austin\'s Wiki\nwords words words";
-        let markdown_page = MarkdownPage::new(&settings, "file_name", &input);
-        assert_eq!("Austin\u{2019}s Wiki", markdown_page.title());
-        let rendered = markdown_page.render_html();
-        assert_eq!("<p>words words words</p>\n", rendered);
-    }
-
     fn assert_request_path_parse(
         input: &[&'static str],
         expected_file_stem: &str,
@@ -374,4 +380,15 @@ mod tests {
         assert!(extensionless_file.directories().is_empty());
         assert!(extensionless_file.file_name_and_extension().is_none());
     }
+
+    #[test]
+    fn test_wikipath_append() {
+        let empty = WikiPagePath::new(vec![]);
+
+        let folder = empty.append_segment("folder");
+        assert_eq!(folder.segments, vec!["folder"]);
+
+        let file = folder.append_segment("file");
+        assert_eq!(file.segments, vec!["folder", "file"]);
+    }
 }
